import java.util.ArrayList;
import java.util.List;

import org.drools.compiler.CommonTestMethodBase;
import org.junit.Test;
import org.kie.api.KieBase;
import org.kie.api.runtime.KieSession;

public class DroolsMathTest extends CommonTestMethodBase {

	private static final String RULE_WITHOUT_PARENTHESIS = 
			"dialect \"mvel\"\n" +
			"import DroolsMathTest$MyFact;\n" +
			"global java.util.List list;\n" +
			"\n" +
			"rule \"calculate-without-parenthesis\"\n" +
			"when\n" +
			"   $fact : MyFact()\n" +
			"then\n" +
			   "list.add(  0.50 * $fact.Source_In * $fact.In_MF_Fkt_ESP * (1/(($fact.Q100_In_norm_ESP + $fact.Q50_In_Offset_ESP) * 0.50)-1/($fact.Q100_In_norm_ESP + $fact.Q50_In_Offset_ESP)- 1 / 0.50 + 1) / $fact.Ga2_MF_Fkt_ESP );\n" +
			"end\n";

	private static final String RULE_WITH_PARENTHESIS = 
			"dialect \"mvel\"\n" +
			"import DroolsMathTest$MyFact;\n" +
			"global java.util.List list;\n" +
			"\n" +
			"rule \"calculate-without-parenthesis\"\n" +
			"when\n" +
			"   $fact : MyFact()\n" +
			"then\n" +
			   "list.add(  0.50 * $fact.Source_In * $fact.In_MF_Fkt_ESP * (1/(($fact.Q100_In_norm_ESP + $fact.Q50_In_Offset_ESP) * 0.50)-(1/($fact.Q100_In_norm_ESP + $fact.Q50_In_Offset_ESP)- 1 / 0.50 + 1) / $fact.Ga2_MF_Fkt_ESP) );\n" +
			"end\n";


	@Test
	public void testCalculation() throws Throwable {
        KieBase kbase = loadKnowledgeBaseFromString( RULE_WITHOUT_PARENTHESIS );
        KieSession ksession = kbase.newKieSession();
        MyFact fact = new MyFact();
        ksession.insert(fact);
        List<Double> list = new ArrayList<Double>();
        ksession.setGlobal("list", list);

        assertEquals( 1, ksession.fireAllRules() );
		assertEquals(fact.getResult(), list.get(0).doubleValue(), 0.1d);
	}
	
	@Test
	public void testCalculationWithParenthesis() throws Throwable {
		KieBase kbase = loadKnowledgeBaseFromString( RULE_WITH_PARENTHESIS );
		KieSession ksession = kbase.newKieSession();
        MyFact fact = new MyFact();
        ksession.insert(fact);
        List<Double> list = new ArrayList<Double>();
        ksession.setGlobal("list", list);

        assertEquals( 1, ksession.fireAllRules() );
		assertEquals(fact.getResultWithParenthesis(), list.get(0).doubleValue(), 0.1d);
		
	}


	public class MyFact {

		public double Source_In = 660d;
		public double In_MF_Fkt_ESP = 0.445048d;
		public double Q100_In_norm_ESP = 0.6d;
		public double Q50_In_Offset_ESP = -0.011d;
		public double Ga2_MF_Fkt_ESP = 7.665088d;	   


		public double getResult() {
			return  0.50 * Source_In * In_MF_Fkt_ESP * (1/((Q100_In_norm_ESP + Q50_In_Offset_ESP) * 0.50)-1/(Q100_In_norm_ESP + Q50_In_Offset_ESP)- 1 / 0.50 + 1) / Ga2_MF_Fkt_ESP;
		}

		public double getResultWithParenthesis() {
			return  0.50 * Source_In * In_MF_Fkt_ESP * (1/((Q100_In_norm_ESP + Q50_In_Offset_ESP) * 0.50)-(1/(Q100_In_norm_ESP + Q50_In_Offset_ESP)- 1 / 0.50 + 1) / Ga2_MF_Fkt_ESP);
		}	   
	}

}
